"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/**
 * This metadata contains all information about entity's listeners.
 */
var EntityListenerMetadata = /** @class */ (function () {
    // ---------------------------------------------------------------------
    // Constructor
    // ---------------------------------------------------------------------
    function EntityListenerMetadata(options) {
        this.entityMetadata = options.entityMetadata;
        this.embeddedMetadata = options.embeddedMetadata;
        this.target = options.args.target;
        this.propertyName = options.args.propertyName;
        this.type = options.args.type;
    }
    // ---------------------------------------------------------------------
    // Public Methods
    // ---------------------------------------------------------------------
    /**
     * Checks if entity listener is allowed to be executed on the given entity.
     */
    EntityListenerMetadata.prototype.isAllowed = function (entity) {
        return this.entityMetadata.target === entity.constructor || // todo: .constructor won't work for entity schemas, but there are no entity listeners in schemas since there are no objects, right?
            (this.entityMetadata.target instanceof Function && entity.constructor.prototype instanceof this.entityMetadata.target); // todo: also need to implement entity schema inheritance
    };
    /**
     * Executes listener method of the given entity.
     */
    EntityListenerMetadata.prototype.execute = function (entity) {
        var entityEmbeddedCallbackParams = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            entityEmbeddedCallbackParams[_i - 1] = arguments[_i];
        }
        if (!this.embeddedMetadata)
            return entity[this.propertyName].apply(entity, tslib_1.__spread(entityEmbeddedCallbackParams));
        this.callEntityEmbeddedMethod.apply(this, tslib_1.__spread([entity, this.embeddedMetadata.propertyPath.split(".")], entityEmbeddedCallbackParams));
    };
    // ---------------------------------------------------------------------
    // Protected Methods
    // ---------------------------------------------------------------------
    /**
     * Calls embedded entity listener method no matter how nested it is.
     */
    EntityListenerMetadata.prototype.callEntityEmbeddedMethod = function (entity, propertyPaths) {
        var entityEmbeddedCallbackParams = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            entityEmbeddedCallbackParams[_i - 2] = arguments[_i];
        }
        var _a;
        var propertyPath = propertyPaths.shift();
        if (!propertyPath || !entity[propertyPath])
            return;
        if (propertyPaths.length === 0) {
            (_a = entity[propertyPath])[this.propertyName].apply(_a, tslib_1.__spread(entityEmbeddedCallbackParams));
        }
        else {
            if (entity[propertyPath])
                this.callEntityEmbeddedMethod.apply(this, tslib_1.__spread([entity[propertyPath], propertyPaths], entityEmbeddedCallbackParams));
        }
    };
    return EntityListenerMetadata;
}());
exports.EntityListenerMetadata = EntityListenerMetadata;

//# sourceMappingURL=EntityListenerMetadata.js.map
